@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2009 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
- * Copyright (C) 2010-2012 Jiri Techet <techet@gmail.com>
+ * Copyright (C) 2010-2013 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -161,20 +161,6 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
         NULL);
   champlain_map_source_factory_register (factory, desc);
 
-  desc = champlain_map_source_desc_new_full (
-        CHAMPLAIN_MAP_SOURCE_OSM_OSMARENDER,
-        "OpenStreetMap Osmarender",
-        "Map data is CC-BY-SA 2.0 OpenStreetMap contributors",
-        "http://creativecommons.org/licenses/by-sa/2.0/",
-        0,
-        17,
-        256,
-        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
-        "http://a.tah.openstreetmap.org/Tiles/tile/#Z#/#X#/#Y#.png",
-        champlain_map_source_new_generic,
-        NULL);
-  champlain_map_source_factory_register (factory, desc);
-
   desc = champlain_map_source_desc_new_full (
         CHAMPLAIN_MAP_SOURCE_OSM_MAPQUEST,
         "MapQuest OSM",
@@ -189,6 +175,20 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
         NULL);
   champlain_map_source_factory_register (factory, desc);
 
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OSM_AERIAL_MAP,
+        "MapQuest Open Aerial",
+        "Map data is CC-BY-SA 2.0 OpenStreetMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        18,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://otile1.mqcdn.com/tiles/1.0.0/sat/#Z#/#X#/#Y#.jpg",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
+
   desc = champlain_map_source_desc_new_full (
         CHAMPLAIN_MAP_SOURCE_OSM_CYCLE_MAP,
         "OpenStreetMap Cycle Map",
@@ -217,8 +217,8 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
         NULL);
   champlain_map_source_factory_register (factory, desc);
 
+/* Not available any more - remove completely in the next release */
 #if 0
-/* Disabling until OpenArealMap works again */
   desc = champlain_map_source_desc_new_full (
         CHAMPLAIN_MAP_SOURCE_OAM,
         "OpenAerialMap",
@@ -232,6 +232,20 @@ champlain_map_source_factory_init (ChamplainMapSourceFactory *factory)
         champlain_map_source_new_generic,
         NULL);
   champlain_map_source_factory_register (factory, desc);
+
+  desc = champlain_map_source_desc_new_full (
+        CHAMPLAIN_MAP_SOURCE_OSM_OSMARENDER,
+        "OpenStreetMap Osmarender",
+        "Map data is CC-BY-SA 2.0 OpenStreetMap contributors",
+        "http://creativecommons.org/licenses/by-sa/2.0/",
+        0,
+        17,
+        256,
+        CHAMPLAIN_MAP_PROJECTION_MERCATOR,
+        "http://a.tah.openstreetmap.org/Tiles/tile/#Z#/#X#/#Y#.png",
+        champlain_map_source_new_generic,
+        NULL);
+  champlain_map_source_factory_register (factory, desc);
 #endif
 
   desc = champlain_map_source_desc_new_full (
