@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008-2009 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
- * Copyright (C) 2010-2012 Jiri Techet <techet@gmail.com>
+ * Copyright (C) 2010-2013 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -74,7 +74,6 @@ struct _ChamplainTilePrivate
   ChamplainState state; /* The tile state: loading, validation, done */
   /* The tile actor that will be displayed after champlain_tile_display_content () */
   ClutterActor *content_actor;
-  ClutterGroup *content_group; /* A group used for the fade in effect */
   gboolean fade_in;
 
   GTimeVal *modified_time; /* The last modified time of the cache */
@@ -178,99 +177,6 @@ champlain_tile_set_property (GObject *object,
 }
 
 
-static void
-paint (ClutterActor *self)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  clutter_actor_paint (CLUTTER_ACTOR (priv->content_group));
-}
-
-
-static void
-pick (ClutterActor *self,
-    const ClutterColor *color)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->pick (self, color);
-
-  clutter_actor_paint (CLUTTER_ACTOR (priv->content_group));
-}
-
-
-static void
-get_preferred_width (ClutterActor *self,
-    gfloat for_height,
-    gfloat *min_width_p,
-    gfloat *natural_width_p)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  clutter_actor_get_preferred_width (CLUTTER_ACTOR (priv->content_group),
-      for_height,
-      min_width_p,
-      natural_width_p);
-}
-
-
-static void
-get_preferred_height (ClutterActor *self,
-    gfloat for_width,
-    gfloat *min_height_p,
-    gfloat *natural_height_p)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  clutter_actor_get_preferred_height (CLUTTER_ACTOR (priv->content_group),
-      for_width,
-      min_height_p,
-      natural_height_p);
-}
-
-
-static void
-allocate (ClutterActor *self,
-    const ClutterActorBox *box,
-    ClutterAllocationFlags flags)
-{
-  ClutterActorBox child_box;
-
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->allocate (self, box, flags);
-
-  child_box.x1 = 0;
-  child_box.x2 = box->x2 - box->x1;
-  child_box.y1 = 0;
-  child_box.y2 = box->y2 - box->y1;
-
-  clutter_actor_allocate (CLUTTER_ACTOR (priv->content_group), &child_box, flags);
-}
-
-
-static void
-map (ClutterActor *self)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->map (self);
-
-  clutter_actor_map (CLUTTER_ACTOR (priv->content_group));
-}
-
-
-static void
-unmap (ClutterActor *self)
-{
-  ChamplainTilePrivate *priv = GET_PRIVATE (self);
-
-  CLUTTER_ACTOR_CLASS (champlain_tile_parent_class)->unmap (self);
-
-  clutter_actor_unmap (CLUTTER_ACTOR (priv->content_group));
-}
-
-
 static void
 champlain_tile_dispose (GObject *object)
 {
@@ -282,12 +188,6 @@ champlain_tile_dispose (GObject *object)
       priv->content_actor = NULL;
     }
 
-  if (priv->content_group)
-    {
-      clutter_actor_unparent (CLUTTER_ACTOR (priv->content_group));
-      priv->content_group = NULL;
-    }
-
   G_OBJECT_CLASS (champlain_tile_parent_class)->dispose (object);
 }
 
@@ -308,7 +208,6 @@ static void
 champlain_tile_class_init (ChamplainTileClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (ChamplainTilePrivate));
 
@@ -317,14 +216,6 @@ champlain_tile_class_init (ChamplainTileClass *klass)
   object_class->dispose = champlain_tile_dispose;
   object_class->finalize = champlain_tile_finalize;
 
-  actor_class->get_preferred_width = get_preferred_width;
-  actor_class->get_preferred_height = get_preferred_height;
-  actor_class->allocate = allocate;
-  actor_class->paint = paint;
-  actor_class->pick = pick;
-  actor_class->map = map;
-  actor_class->unmap = unmap;
-
   /**
    * ChamplainTile:x:
    *
@@ -501,9 +392,6 @@ champlain_tile_init (ChamplainTile *self)
   priv->content_displayed = FALSE;
 
   priv->content_actor = NULL;
-  priv->content_group = CLUTTER_GROUP (clutter_group_new ());
-  clutter_actor_set_parent (CLUTTER_ACTOR (priv->content_group), CLUTTER_ACTOR (self));
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (self));
 }
 
 
@@ -858,9 +746,6 @@ champlain_tile_set_content (ChamplainTile *self,
 
   ChamplainTilePrivate *priv = self->priv;
 
-  if (!priv->content_group)
-    return;
-
   if (!priv->content_displayed && priv->content_actor)
     clutter_actor_destroy (priv->content_actor);
 
@@ -872,15 +757,15 @@ champlain_tile_set_content (ChamplainTile *self,
 
 
 static void
-fade_in_completed (G_GNUC_UNUSED ClutterAnimation *animation, ChamplainTile *self)
+fade_in_completed (ClutterActor *actor,
+    const gchar *transition_name,
+    gboolean is_finished,
+    ChamplainTile *self)
 {
-  ChamplainTilePrivate *priv = self->priv;
-  
-  if (!priv->content_group)
-    return;
+  if (clutter_actor_get_n_children (CLUTTER_ACTOR (self)) > 1)
+    clutter_actor_destroy (clutter_actor_get_first_child (CLUTTER_ACTOR (self)));
 
-  if (clutter_group_get_n_children (CLUTTER_GROUP (priv->content_group)) > 1)
-    clutter_actor_destroy (clutter_group_get_nth_child (CLUTTER_GROUP (priv->content_group), 0));
+  g_signal_handlers_disconnect_by_func (actor, fade_in_completed, self);
 }
 
 
@@ -898,34 +783,30 @@ champlain_tile_display_content (ChamplainTile *self)
   g_return_if_fail (CHAMPLAIN_TILE (self));
 
   ChamplainTilePrivate *priv = self->priv;
-  ClutterAnimation *animation;
 
-  if (!priv->content_actor || priv->content_displayed || !priv->content_group)
+  if (!priv->content_actor || priv->content_displayed)
     return;
 
-  clutter_container_add_actor (CLUTTER_CONTAINER (priv->content_group), priv->content_actor);
+  clutter_actor_add_child (CLUTTER_ACTOR (self), priv->content_actor);
   g_object_unref (priv->content_actor);
   priv->content_displayed = TRUE;
 
-  clutter_actor_set_opacity (CLUTTER_ACTOR (priv->content_actor), 0);
+  clutter_actor_set_opacity (priv->content_actor, 0);
+  clutter_actor_save_easing_state (priv->content_actor);
   if (priv->fade_in)
     {
-      animation = clutter_actor_animate (CLUTTER_ACTOR (priv->content_actor),
-            CLUTTER_EASE_IN_CUBIC,
-            500,
-            "opacity", 255,
-            NULL);
+      clutter_actor_set_easing_mode (priv->content_actor, CLUTTER_EASE_IN_CUBIC);
+      clutter_actor_set_easing_duration (priv->content_actor, 500);
     }
   else
     {
-      animation = clutter_actor_animate (CLUTTER_ACTOR (priv->content_actor),
-            CLUTTER_LINEAR,
-            150,
-            "opacity", 255,
-            NULL);
+      clutter_actor_set_easing_mode (priv->content_actor, CLUTTER_LINEAR);
+      clutter_actor_set_easing_duration (priv->content_actor, 150);
     }
+  clutter_actor_set_opacity (priv->content_actor, 255);
+  clutter_actor_restore_easing_state (priv->content_actor);
 
-  g_signal_connect (animation, "completed", G_CALLBACK (fade_in_completed), self);
+  g_signal_connect (priv->content_actor, "transition-stopped::opacity", G_CALLBACK (fade_in_completed), self);
 }
 
 
