@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2008 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
- * Copyright (C) 2010-2012 Jiri Techet <techet@gmail.com>
+ * Copyright (C) 2010-2013 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -117,6 +117,8 @@ void champlain_view_set_zoom_on_double_click (ChamplainView *view,
     gboolean value);
 void champlain_view_set_animate_zoom (ChamplainView *view,
     gboolean value);
+void champlain_view_set_background_pattern (ChamplainView *view,
+    ClutterContent *background);
 
 void champlain_view_add_layer (ChamplainView *view,
     ChamplainLayer *layer);
@@ -133,6 +135,7 @@ gboolean champlain_view_get_keep_center_on_resize (ChamplainView *view);
 gboolean champlain_view_get_zoom_on_double_click (ChamplainView *view);
 gboolean champlain_view_get_animate_zoom (ChamplainView *view);
 ChamplainState champlain_view_get_state (ChamplainView *view);
+ClutterContent *champlain_view_get_background_pattern (ChamplainView *view);
 
 void champlain_view_reload_tiles (ChamplainView *view);
 
@@ -149,13 +152,16 @@ void champlain_view_get_viewport_origin (ChamplainView *view,
     gint *x,
     gint *y);
 
+#ifndef GTK_DISABLE_DEPRECATED
 void champlain_view_bin_layout_add (ChamplainView *view,
     ClutterActor *child,
     ClutterBinAlignment x_align,
     ClutterBinAlignment y_align);
-
+#endif
 ChamplainLicense *champlain_view_get_license_actor (ChamplainView *view);
 
+ChamplainBoundingBox *champlain_view_get_bounding_box (ChamplainView *view);
+
 G_END_DECLS
 
 #endif
